@@ -5,14 +5,12 @@ import os
 import time
 
 from tqdm import tqdm
-from typing import Dict
+from typing import Dict, Optional, List
 
-from sahi.postprocess.match import PredictionMatcher
-from sahi.postprocess.merge import PredictionMerger, ScoreMergingPolicy
-from sahi.postprocess.ops import box_ios, box_union
-from sahi.prediction import ObjectPrediction, PredictionInput
+from sahi.prediction import ObjectPrediction
+from sahi.postprocess.combine import UnionMergePostprocess, PostprocessPredictions, NMSPostprocess
 from sahi.slicing import slice_image
-from sahi.utils.coco import Coco
+from sahi.utils.coco import Coco, CocoImage
 from sahi.utils.cv import (
     crop_object_predictions,
     read_image,
@@ -33,8 +31,7 @@ def get_prediction(
     detection_model,
     shift_amount: list = [0, 0],
     full_shape=None,
-    merger=None,
-    matcher=None,
+    postprocess: Optional[PostprocessPredictions] = None,
     verbose: int = 0,
 ):
     """
@@ -49,8 +46,7 @@ def get_prediction(
             sized image, should be in the form of [shift_x, shift_y]
         full_shape: List
             Size of the full image, should be in the form of [height, width]
-        merger: postprocess.PredictionMerger
-        matcher: postprocess.PredictionMatcher
+        postprocess: sahi.postprocess.combine.PostprocessPredictions
         verbose: int
             0: no print (default)
             1: print prediction duration
@@ -78,33 +74,21 @@ def get_prediction(
         shift_amount=shift_amount,
         full_shape=full_shape,
     )
-    object_prediction_list = detection_model.object_prediction_list
+    object_prediction_list: List[ObjectPrediction] = detection_model.object_prediction_list
     # filter out predictions with lower score
     filtered_object_prediction_list = [
         object_prediction
         for object_prediction in object_prediction_list
-        if object_prediction.score.score > detection_model.prediction_score_threshold
+        if object_prediction.score.value > detection_model.prediction_score_threshold
     ]
-    # merge matching predictions
-    if merger is not None:
-        filtered_object_prediction_list = merger.merge_batch(
-            matcher,
-            filtered_object_prediction_list,
-            merge_type="merge",
-        )
+    # postprocess matching predictions
+    if postprocess is not None:
+        filtered_object_prediction_list = postprocess(filtered_object_prediction_list)
     else:
         # init match merge instances
-        merger = PredictionMerger(
-            score_merging=ScoreMergingPolicy.LARGER_SCORE, box_merger=box_union
-        )
-        matcher = PredictionMatcher(threshold=0.5, scorer=box_ios)
-        # merge matching predictions
-        filtered_object_prediction_list = merger.merge_batch(
-            matcher,
-            filtered_object_prediction_list,
-            merge_type="merge",
-            ignore_class_label=True,
-        )
+        postprocess = UnionMergePostprocess(match_threshold=0.9, match_metric="IOS", class_agnostic=True)
+        # postprocess matching predictions
+        filtered_object_prediction_list = postprocess(filtered_object_prediction_list)
 
     time_end = time.time() - time_start
     durations_in_seconds["postprocess"] = time_end
@@ -129,7 +113,10 @@ def get_sliced_prediction(
     slice_width: int = 256,
     overlap_height_ratio: float = 0.2,
     overlap_width_ratio: float = 0.2,
-    match_iou_threshold: float = 0.5,
+    postprocess_type: str = "UNIONMERGE",
+    postprocess_match_metric: str = "IOS",
+    postprocess_match_threshold: float = 0.5,
+    postprocess_class_agnostic: bool = False,
     verbose: int = 1,
 ):
     """
@@ -151,8 +138,17 @@ def get_sliced_prediction(
             Fractional overlap in width of each window (e.g. an overlap of 0.2 for a window
             of size 256 yields an overlap of 51 pixels).
             Default to ``0.2``.
-        match_iou_threshold: float
-            Sliced predictions having higher iou than match_iou_threshold will be merged.
+        postprocess_type: str
+            Type of the postprocess to be used after sliced inference while merging/eliminating predictions.
+            Options are 'UNIONMERGE' or 'NMS'. Default is 'UNIONMERGE'.
+        postprocess_match_metric: str
+            Metric to be used during object prediction matching after sliced prediction.
+            'IOU' for intersection over union, 'IOS' for intersection over smaller area.
+        postprocess_match_threshold: float
+            Sliced predictions having higher iou than postprocess_match_threshold will be
+            postprocessed after sliced prediction.
+        postprocess_class_agnostic: bool
+            If True, postprocess will ignore category ids.
         verbose: int
             0: no print
             1: print number of slices (default)
@@ -182,11 +178,21 @@ def get_sliced_prediction(
     time_end = time.time() - time_start
     durations_in_seconds["slice"] = time_end
 
-    # init match merge instances
-    merger = PredictionMerger(
-        score_merging=ScoreMergingPolicy.LARGER_SCORE, box_merger=box_union
-    )
-    matcher = PredictionMatcher(threshold=match_iou_threshold, scorer=box_ios)
+    # init match postprocess instance
+    if postprocess_type == "UNIONMERGE":
+        postprocess = UnionMergePostprocess(
+            match_threshold=postprocess_match_threshold,
+            match_metric=postprocess_match_metric,
+            class_agnostic=postprocess_class_agnostic,
+        )
+    elif postprocess_type == "NMS":
+        postprocess = NMSPostprocess(
+            match_threshold=postprocess_match_threshold,
+            match_metric=postprocess_match_metric,
+            class_agnostic=postprocess_class_agnostic,
+        )
+    else:
+        raise ValueError(f"postprocess_type should be one of ['UNIOUNMERGE', 'NMS'] but given as {postprocess_type}")
 
     # create prediction input
     num_group = int(num_slices / num_batch)
@@ -202,14 +208,8 @@ def get_sliced_prediction(
             image_list = []
             shift_amount_list = []
             for image_ind in range(num_batch):
-                image_list.append(
-                    slice_image_result.images[group_ind * num_batch + image_ind]
-                )
-                shift_amount_list.append(
-                    slice_image_result.starting_pixels[
-                        group_ind * num_batch + image_ind
-                    ]
-                )
+                image_list.append(slice_image_result.images[group_ind * num_batch + image_ind])
+                shift_amount_list.append(slice_image_result.starting_pixels[group_ind * num_batch + image_ind])
             # perform batch prediction
             prediction_result = get_prediction(
                 image=image_list[0],
@@ -227,18 +227,13 @@ def get_sliced_prediction(
             detection_model=detection_model,
             shift_amount=[0, 0],
             full_shape=None,
-            merger=None,
-            matcher=None,
+            match_threshold=None,
             verbose=0,
         )
         object_prediction_list.extend(prediction_result["object_prediction_list"])
 
     # remove empty predictions
-    object_prediction_list = [
-        object_prediction
-        for object_prediction in object_prediction_list
-        if object_prediction
-    ]
+    object_prediction_list = [object_prediction for object_prediction in object_prediction_list if object_prediction]
     # convert sliced predictions to full predictions
     full_object_prediction_list = []
     for object_prediction in object_prediction_list:
@@ -261,12 +256,7 @@ def get_sliced_prediction(
         )
 
     # merge matching predictions
-    full_object_prediction_list = merger.merge_batch(
-        matcher,
-        full_object_prediction_list,
-        merge_type="merge",
-    )
-
+    full_object_prediction_list = postprocess(full_object_prediction_list)
     # return filtered elements of filtered full_object_prediction_list
     return {
         "object_prediction_list": full_object_prediction_list,
@@ -283,7 +273,10 @@ def predict(
     slice_width: int = 256,
     overlap_height_ratio: float = 0.2,
     overlap_width_ratio: float = 0.2,
-    match_iou_threshold: float = 0.5,
+    postprocess_type: str = "UNIONMERGE",
+    postprocess_match_metric: str = "IOS",
+    postprocess_match_threshold: float = 0.5,
+    postprocess_class_agnostic: bool = False,
     export_visual: bool = True,
     export_pickle: bool = False,
     export_crop: bool = False,
@@ -329,8 +322,17 @@ def predict(
             Fractional overlap in width of each window (e.g. an overlap of 0.2 for a window
             of size 256 yields an overlap of 51 pixels).
             Default to ``0.2``.
-        match_iou_threshold: float
-            Sliced predictions having higher iou than match_iou_threshold will be merged.
+        postprocess_type: str
+            Type of the postprocess to be used after sliced inference while merging/eliminating predictions.
+            Options are 'UNIONMERGE' or 'NMS'. Default is 'UNIONMERGE'.
+        postprocess_match_metric: str
+            Metric to be used during object prediction matching after sliced prediction.
+            'IOU' for intersection over union, 'IOS' for intersection over smaller area.
+        postprocess_match_threshold: float
+            Sliced predictions having higher iou than postprocess_match_threshold will be
+            postprocessed after sliced prediction.
+        postprocess_class_agnostic: bool
+            If True, postprocess will ignore category ids.
         export_pickle: bool
             Export predictions as .pickle
         export_crop: bool
@@ -355,11 +357,8 @@ def predict(
 
     # list image files in directory
     if coco_file_path:
-        coco = Coco.from_coco_dict_or_path(coco_file_path)
-        image_path_list = [
-            str(Path(source) / Path(coco_image.file_name)) for coco_image in coco.images
-        ]
-        image_id_list = [coco_image.id for coco_image in coco.images]
+        coco: Coco = Coco.from_coco_dict_or_path(coco_file_path)
+        image_path_list = [str(Path(source) / Path(coco_image.file_name)) for coco_image in coco.images]
         coco_json = []
     elif os.path.isdir(source):
         time_start = time.time()
@@ -375,11 +374,10 @@ def predict(
         durations_in_seconds["list_files"] = 0
 
     # init export directories
-    save_dir = Path(
-        increment_path(Path(project) / name, exist_ok=False)
-    )  # increment run
+    save_dir = Path(increment_path(Path(project) / name, exist_ok=False))  # increment run
     crop_dir = save_dir / "crops"
     visual_dir = save_dir / "visuals"
+    visual_with_gt_dir = save_dir / "visuals_with_gt"
     pickle_dir = save_dir / "pickles"
     save_dir.mkdir(parents=True, exist_ok=True)  # make dir
 
@@ -389,7 +387,7 @@ def predict(
     detection_model = DetectionModel(
         model_path=model_parameters["model_path"],
         config_path=model_parameters.get("config_path", None),
-        prediction_score_threshold=model_parameters["prediction_score_threshold"],
+        prediction_score_threshold=model_parameters.get("prediction_score_threshold", 0.25),
         device=model_parameters.get("device", None),
         category_mapping=model_parameters.get("category_mapping", None),
         category_remapping=model_parameters.get("category_remapping", None),
@@ -404,7 +402,12 @@ def predict(
     durations_in_seconds["slice"] = 0
     for ind, image_path in enumerate(tqdm(image_path_list)):
         # get filename
-        filename_without_extension = str(Path(image_path).stem)
+        if os.path.isdir(source):  # preserve source folder structure in export
+            relative_filepath = image_path.split(source)[-1]
+            relative_filepath = relative_filepath[1:] if relative_filepath[0] == os.sep else relative_filepath
+        else:  # no process if source is single file
+            relative_filepath = image_path
+        filename_without_extension = Path(relative_filepath).stem
         # load image
         image = read_image(image_path)
 
@@ -418,13 +421,14 @@ def predict(
                 slice_width=slice_width,
                 overlap_height_ratio=overlap_height_ratio,
                 overlap_width_ratio=overlap_width_ratio,
-                match_iou_threshold=match_iou_threshold,
+                postprocess_type=postprocess_type,
+                postprocess_match_metric=postprocess_match_metric,
+                postprocess_match_threshold=postprocess_match_threshold,
+                postprocess_class_agnostic=postprocess_class_agnostic,
                 verbose=verbose,
             )
             object_prediction_list = prediction_result["object_prediction_list"]
-            durations_in_seconds["slice"] += prediction_result["durations_in_seconds"][
-                "slice"
-            ]
+            durations_in_seconds["slice"] += prediction_result["durations_in_seconds"]["slice"]
         else:
             # get full sized prediction
             prediction_result = get_prediction(
@@ -432,47 +436,84 @@ def predict(
                 detection_model=detection_model,
                 shift_amount=[0, 0],
                 full_shape=None,
-                merger=None,
-                matcher=None,
+                postprocess=None,
                 verbose=0,
             )
             object_prediction_list = prediction_result["object_prediction_list"]
 
-        durations_in_seconds["prediction"] += prediction_result["durations_in_seconds"][
-            "prediction"
-        ]
+        durations_in_seconds["prediction"] += prediction_result["durations_in_seconds"]["prediction"]
 
         if coco_file_path:
-            image_id = image_id_list[ind]
+            image_id = ind + 1
+            # append predictions in coco format
             for object_prediction in object_prediction_list:
                 coco_prediction = object_prediction.to_coco_prediction()
                 coco_prediction.image_id = image_id
                 coco_prediction_json = coco_prediction.json
                 coco_json.append(coco_prediction_json)
+            # convert ground truth annotations to object_prediction_list
+            coco_image: CocoImage = coco.images[ind]
+            object_prediction_gt_list: List[ObjectPrediction] = []
+            for coco_annotation in coco_image.annotations:
+                coco_annotation_dict = coco_annotation.json
+                category_name = coco_annotation.category_name
+                full_shape = [coco_image.height, coco_image.width]
+                object_prediction_gt = ObjectPrediction.from_coco_annotation_dict(
+                    annotation_dict=coco_annotation_dict, category_name=category_name, full_shape=full_shape
+                )
+                object_prediction_gt_list.append(object_prediction_gt)
+            # export visualizations with ground truths
+            output_dir = str(visual_with_gt_dir / Path(relative_filepath).parent)
+            color = (0, 255, 0)  # original annotations in green
+            result = visualize_object_predictions(
+                image,
+                object_prediction_list=object_prediction_gt_list,
+                rect_th=visual_bbox_thickness,
+                text_size=visual_text_size,
+                text_th=visual_text_thickness,
+                color=color,
+                output_dir=None,
+                file_name=None,
+                export_format=None,
+            )
+            color = (255, 0, 0)  # model predictions in red
+            _ = visualize_object_predictions(
+                result["image"],
+                object_prediction_list=object_prediction_list,
+                rect_th=visual_bbox_thickness,
+                text_size=visual_text_size,
+                text_th=visual_text_thickness,
+                color=color,
+                output_dir=output_dir,
+                file_name=filename_without_extension,
+                export_format=visual_export_format,
+            )
 
         time_start = time.time()
         # export prediction boxes
         if export_crop:
+            output_dir = str(crop_dir / Path(relative_filepath).parent)
             crop_object_predictions(
                 image=image,
                 object_prediction_list=object_prediction_list,
-                output_dir=str(crop_dir),
+                output_dir=output_dir,
                 file_name=filename_without_extension,
                 export_format=visual_export_format,
             )
         # export prediction list as pickle
         if export_pickle:
-            save_path = str(pickle_dir / (filename_without_extension + ".pickle"))
+            save_path = str(pickle_dir / Path(relative_filepath).parent / (filename_without_extension + ".pickle"))
             save_pickle(data=object_prediction_list, save_path=save_path)
         # export visualization
         if export_visual:
+            output_dir = str(visual_dir / Path(relative_filepath).parent)
             visualize_object_predictions(
                 image,
                 object_prediction_list=object_prediction_list,
                 rect_th=visual_bbox_thickness,
                 text_size=visual_text_size,
                 text_th=visual_text_thickness,
-                output_dir=str(visual_dir),
+                output_dir=output_dir,
                 file_name=filename_without_extension,
                 export_format=visual_export_format,
             )
