@@ -4,14 +4,14 @@
 
 import copy
 import os
-from collections import Counter, OrderedDict, defaultdict
+from collections import Counter, defaultdict
 from dataclasses import dataclass
 from multiprocessing import Pool
 from pathlib import Path
 from typing import Dict, List, Optional, Set, Union
 
 import numpy as np
-from sahi.utils.file import get_base_filename, load_json, save_json
+from sahi.utils.file import load_json, save_json
 from sahi.utils.shapely import ShapelyAnnotation, box, get_shapely_multipolygon
 from tqdm import tqdm
 
@@ -62,9 +62,7 @@ class CocoAnnotation:
     """
 
     @classmethod
-    def from_coco_segmentation(
-        cls, segmentation, category_id, category_name, iscrowd=0
-    ):
+    def from_coco_segmentation(cls, segmentation, category_id, category_name, iscrowd=0):
         """
         Creates CocoAnnotation object using coco segmentation.
 
@@ -108,9 +106,7 @@ class CocoAnnotation:
         )
 
     @classmethod
-    def from_coco_annotation_dict(
-        cls, annotation_dict: Dict, category_name: Optional[str] = None
-    ):
+    def from_coco_annotation_dict(cls, annotation_dict: Dict, category_name: Optional[str] = None):
         """
         Creates CocoAnnotation object from category name and COCO formatted
         annotation dict (with fields "bbox", "segmentation", "category_id").
@@ -196,20 +192,14 @@ class CocoAnnotation:
         self._iscrowd = iscrowd
 
         if self._segmentation:
-            shapely_annotation = ShapelyAnnotation.from_coco_segmentation(
-                segmentation=self._segmentation
-            )
+            shapely_annotation = ShapelyAnnotation.from_coco_segmentation(segmentation=self._segmentation)
         else:
             shapely_annotation = ShapelyAnnotation.from_coco_bbox(bbox=bbox)
         self._shapely_annotation = shapely_annotation
 
     def get_sliced_coco_annotation(self, slice_bbox: List[int]):
-        shapely_polygon = box(
-            slice_bbox[0], slice_bbox[1], slice_bbox[2], slice_bbox[3]
-        )
-        intersection_shapely_annotation = self._shapely_annotation.get_intersection(
-            shapely_polygon
-        )
+        shapely_polygon = box(slice_bbox[0], slice_bbox[1], slice_bbox[2], slice_bbox[3])
+        intersection_shapely_annotation = self._shapely_annotation.get_intersection(shapely_polygon)
         return CocoAnnotation.from_shapely_annotation(
             intersection_shapely_annotation,
             category_id=self.category_id,
@@ -582,9 +572,7 @@ class CocoImage:
         annotation : CocoAnnotation
         """
 
-        assert isinstance(
-            annotation, CocoAnnotation
-        ), "annotation must be a CocoAnnotation instance"
+        assert isinstance(annotation, CocoAnnotation), "annotation must be a CocoAnnotation instance"
         self.annotations.append(annotation)
 
     @property
@@ -637,9 +625,7 @@ class CocoVidImage(CocoImage):
             video_id: int
                 Video id
         """
-        super(CocoVidImage, self).__init__(
-            file_name=file_name, height=height, width=width, id=id
-        )
+        super(CocoVidImage, self).__init__(file_name=file_name, height=height, width=width, id=id)
         self.frame_id = frame_id
         self.video_id = video_id
 
@@ -670,9 +656,7 @@ class CocoVidImage(CocoImage):
         annotation : CocoVidAnnotation
         """
 
-        assert (
-            type(annotation) == CocoVidAnnotation
-        ), "annotation must be a CocoVidAnnotation instance"
+        assert type(annotation) == CocoVidAnnotation, "annotation must be a CocoVidAnnotation instance"
         self.annotations.append(annotation)
 
     @property
@@ -751,9 +735,7 @@ class CocoVideo:
             cocovidimage: CocoVidImage
         """
 
-        assert (
-            type(cocovidimage) == CocoVidImage
-        ), "cocovidimage must be a CocoVidImage instance"
+        assert type(cocovidimage) == CocoVidImage, "cocovidimage must be a CocoVidImage instance"
 
         self.images.append(cocovidimage)
 
@@ -835,9 +817,7 @@ class Coco:
             category: CocoCategory
         """
 
-        assert (
-            type(category) == CocoCategory
-        ), "category must be a CocoCategory instance"
+        assert type(category) == CocoCategory, "category must be a CocoCategory instance"
 
         self.categories.append(category)
 
@@ -877,33 +857,23 @@ class Coco:
             current_category_id = coco_category.id
             current_category_name = coco_category.name
             if current_category_name in desired_name2id.keys():
-                currentid2desiredid_mapping[current_category_id] = desired_name2id[
-                    current_category_name
-                ]
+                currentid2desiredid_mapping[current_category_id] = desired_name2id[current_category_name]
             else:
                 # ignore categories that are not included in desired_name2id
                 currentid2desiredid_mapping[current_category_id] = None
 
         # add updated categories
         for name in desired_name2id.keys():
-            updated_coco_category = CocoCategory(
-                id=desired_name2id[name], name=name, supercategory=name
-            )
+            updated_coco_category = CocoCategory(id=desired_name2id[name], name=name, supercategory=name)
             updated_coco.add_category(updated_coco_category)
 
         # add updated images & annotations
         for coco_image in copy.deepcopy(self.images):
             updated_coco_image = CocoImage.from_coco_image_dict(coco_image.json)
             # update filename to abspath
-            file_name_is_abspath = (
-                True
-                if os.path.abspath(coco_image.file_name) == coco_image.file_name
-                else False
-            )
+            file_name_is_abspath = True if os.path.abspath(coco_image.file_name) == coco_image.file_name else False
             if update_image_filenames and not file_name_is_abspath:
-                updated_coco_image.file_name = str(
-                    Path(os.path.abspath(self.image_dir)) / coco_image.file_name
-                )
+                updated_coco_image.file_name = str(Path(os.path.abspath(self.image_dir)) / coco_image.file_name)
             # update annotations
             for coco_annotation in coco_image.annotations:
                 current_category_id = coco_annotation.category_id
@@ -932,9 +902,7 @@ class Coco:
             verbose: bool
                 If True, merging info is printed
         """
-        assert (
-            self.image_dir and coco.image_dir
-        ), "image_dir should be provided for merging."
+        assert self.image_dir and coco.image_dir, "image_dir should be provided for merging."
 
         if verbose:
             if not desired_name2id:
@@ -957,9 +925,7 @@ class Coco:
 
         # update categories and image paths
         for coco in [coco1, coco2]:
-            coco.update_categories(
-                desired_name2id=desired_name2id, update_image_filenames=True
-            )
+            coco.update_categories(desired_name2id=desired_name2id, update_image_filenames=True)
 
         # combine images and categories
         coco1.images.extend(coco2.images)
@@ -1039,9 +1005,7 @@ class Coco:
                 # apply category remapping if remapping_dict is provided
                 if coco.remapping_dict is not None:
                     # apply category remapping (id:id)
-                    category_id = coco.remapping_dict[
-                        coco_annotation_dict["category_id"]
-                    ]
+                    category_id = coco.remapping_dict[coco_annotation_dict["category_id"]]
                     # update category id
                     coco_annotation_dict["category_id"] = category_id
                 else:
@@ -1093,12 +1057,8 @@ class Coco:
         num_images = len(self.images)
         num_negative_images = 0
         num_categories = len(self.json_categories)
-        category_name_to_zero = {
-            category["name"]: 0 for category in self.json_categories
-        }
-        category_name_to_inf = {
-            category["name"]: float("inf") for category in self.json_categories
-        }
+        category_name_to_zero = {category["name"]: 0 for category in self.json_categories}
+        category_name_to_inf = {category["name"]: float("inf") for category in self.json_categories}
         num_images_per_category = copy.deepcopy(category_name_to_zero)
         num_annotations_per_category = copy.deepcopy(category_name_to_zero)
         min_annotation_area_per_category = copy.deepcopy(category_name_to_inf)
@@ -1120,38 +1080,24 @@ class Coco:
                     max_annotation_area = annotation_area
                 if annotation_area < min_annotation_area:
                     min_annotation_area = annotation_area
-                if (
-                    annotation_area
-                    > max_annotation_area_per_category[annotation.category_name]
-                ):
-                    max_annotation_area_per_category[
-                        annotation.category_name
-                    ] = annotation_area
-                if (
-                    annotation_area
-                    < min_annotation_area_per_category[annotation.category_name]
-                ):
-                    min_annotation_area_per_category[
-                        annotation.category_name
-                    ] = annotation_area
+                if annotation_area > max_annotation_area_per_category[annotation.category_name]:
+                    max_annotation_area_per_category[annotation.category_name] = annotation_area
+                if annotation_area < min_annotation_area_per_category[annotation.category_name]:
+                    min_annotation_area_per_category[annotation.category_name] = annotation_area
             # update num_negative_images
             if len(image.annotations) == 0:
                 num_negative_images += 1
             # update num_annotations
             num_annotations += len(image.annotations)
             # update num_images_per_category
-            num_images_per_category = dict(
-                Counter(num_images_per_category) + Counter(image_contains_category)
-            )
+            num_images_per_category = dict(Counter(num_images_per_category) + Counter(image_contains_category))
             # update min&max_num_annotations_in_image
             num_annotations_in_image = len(image.annotations)
             if num_annotations_in_image > max_num_annotations_in_image:
                 max_num_annotations_in_image = num_annotations_in_image
             if num_annotations_in_image < min_num_annotations_in_image:
                 min_num_annotations_in_image = num_annotations_in_image
-        avg_num_annotations_in_image = num_annotations / (
-            num_images - num_negative_images
-        )
+        avg_num_annotations_in_image = num_annotations / (num_images - num_negative_images)
         avg_annotation_area = total_annotation_area / num_annotations
 
         self._stats = {
@@ -1206,9 +1152,7 @@ class Coco:
         train_coco.images = train_images
         train_coco.categories = self.categories
 
-        val_coco = Coco(
-            name=self.name if self.name else "split" + "_val", image_dir=self.image_dir
-        )
+        val_coco = Coco(name=self.name if self.name else "split" + "_val", image_dir=self.image_dir)
         val_coco.images = val_images
         val_coco.categories = self.categories
 
@@ -1240,8 +1184,7 @@ class Coco:
             import yaml
         except ImportError:
             raise ImportError(
-                'Please run "pip install -U pyyaml" '
-                "to install yaml first for yolov5 formatted exporting."
+                'Please run "pip install -U pyyaml" ' "to install yaml first for yolov5 formatted exporting."
             )
 
         # set split_mode
@@ -1328,9 +1271,7 @@ class Coco:
 
         return subsampled_coco
 
-    def get_area_filtered_coco(
-        self, min=0, max=float("inf"), intervals_per_category=None
-    ):
+    def get_area_filtered_coco(self, min=0, max=float("inf"), intervals_per_category=None):
         """
         Filters annotation areas with given min and max values and returns remaining
         images as sahi.utils.coco.Coco object.
@@ -1358,20 +1299,10 @@ class Coco:
         for image in self.images:
             is_valid_image = True
             for annotation in image.annotations:
-                if (
-                    intervals_per_category is not None
-                    and annotation.category_name in intervals_per_category.keys()
-                ):
-                    category_based_min = intervals_per_category[
-                        annotation.category_name
-                    ]["min"]
-                    category_based_max = intervals_per_category[
-                        annotation.category_name
-                    ]["max"]
-                    if (
-                        annotation.area < category_based_min
-                        or annotation.area > category_based_max
-                    ):
+                if intervals_per_category is not None and annotation.category_name in intervals_per_category.keys():
+                    category_based_min = intervals_per_category[annotation.category_name]["min"]
+                    category_based_max = intervals_per_category[annotation.category_name]["max"]
+                    if annotation.area < category_based_min or annotation.area > category_based_max:
                         is_valid_image = False
                 if annotation.area < min or annotation.area > max:
                     is_valid_image = False
@@ -1381,9 +1312,7 @@ class Coco:
         return area_filtered_coco
 
 
-def export_yolov5_images_and_txts_from_coco_object(
-    output_dir, coco, ignore_negative_samples=False, mp=False
-):
+def export_yolov5_images_and_txts_from_coco_object(output_dir, coco, ignore_negative_samples=False, mp=False):
     """
     Creates image symlinks and annotation txts in yolo format from coco dataset.
 
@@ -1402,10 +1331,7 @@ def export_yolov5_images_and_txts_from_coco_object(
     print("generating image symlinks and annotation files for yolov5..."),
     if mp:
         with Pool(processes=48) as pool:
-            args = [
-                (coco_image, coco.image_dir, output_dir, ignore_negative_samples)
-                for coco_image in coco.images
-            ]
+            args = [(coco_image, coco.image_dir, output_dir, ignore_negative_samples) for coco_image in coco.images]
             pool.starmap(
                 export_single_yolov5_image_and_corresponding_txt,
                 tqdm(args, total=len(args)),
@@ -1436,12 +1362,8 @@ def export_single_yolov5_image_and_corresponding_txt(
         if Path(coco_image.file_name).is_file():
             coco_image_path = os.path.abspath(coco_image.file_name)
         else:
-            assert coco_image_dir is not None, (
-                "You have to specify image_dir " "of Coco object for yolov5 conversion."
-            )
-            coco_image_path = os.path.abspath(
-                str(Path(coco_image_dir) / coco_image.file_name)
-            )
+            assert coco_image_dir is not None, "You have to specify image_dir " "of Coco object for yolov5 conversion."
+            coco_image_path = os.path.abspath(str(Path(coco_image_dir) / coco_image.file_name))
         yolo_image_path_temp = str(Path(output_dir) / Path(coco_image.file_name).name)
         # increment target file name if already present
         yolo_image_path = copy.deepcopy(yolo_image_path_temp)
@@ -1478,12 +1400,7 @@ def export_single_yolov5_image_and_corresponding_txt(
                 category_id = annotation.category_id
                 yolo_bbox = (x_center, y_center, bbox_width, bbox_height)
                 # save yolo annotation
-                outfile.write(
-                    str(category_id)
-                    + " "
-                    + " ".join([str(value) for value in yolo_bbox])
-                    + "\n"
-                )
+                outfile.write(str(category_id) + " " + " ".join([str(value) for value in yolo_bbox]) + "\n")
 
 
 def update_categories(desired_name2id: dict, coco_dict: dict) -> dict:
@@ -1515,9 +1432,7 @@ def update_categories(desired_name2id: dict, coco_dict: dict) -> dict:
         current_category_id = category["id"]
         current_category_name = category["name"]
         if current_category_name in desired_name2id.keys():
-            currentid2desiredid_mapping[current_category_id] = desired_name2id[
-                current_category_name
-            ]
+            currentid2desiredid_mapping[current_category_id] = desired_name2id[current_category_name]
         else:
             # ignore categories that are not included in desired_name2id
             currentid2desiredid_mapping[current_category_id] = -1
@@ -1550,9 +1465,7 @@ def update_categories(desired_name2id: dict, coco_dict: dict) -> dict:
     return coco_target
 
 
-def update_categories_from_file(
-    desired_name2id: dict, coco_path: str, save_path: str
-) -> None:
+def update_categories_from_file(desired_name2id: dict, coco_path: str, save_path: str) -> None:
     """
     Rearranges category mapping of a COCO dictionary in coco_path based on given category_mapping.
     Can also be used to filter some of the categories.
@@ -1602,10 +1515,7 @@ def merge(coco_dict1: dict, coco_dict2: dict, desired_name2id: dict = None) -> d
 
     # rearrange categories of the second coco based on first, if their categories are not the same
     if temp_coco_dict1["categories"] != temp_coco_dict2["categories"]:
-        desired_name2id = {
-            category["name"]: category["id"]
-            for category in temp_coco_dict1["categories"]
-        }
+        desired_name2id = {category["name"]: category["id"] for category in temp_coco_dict1["categories"]}
         temp_coco_dict2 = update_categories(desired_name2id, temp_coco_dict2)
 
     # calculate first image and annotation index of the second coco file
@@ -1863,12 +1773,8 @@ def split_coco_as_train_val(
     # divide images
     train_indices = random_indices[:num_train]
     val_indices = random_indices[num_train:]
-    train_images = np.array(coco_dict["images"])[
-        (np.array(train_indices) - 1).tolist()
-    ].tolist()
-    val_images = np.array(coco_dict["images"])[
-        (np.array(val_indices) - 1).tolist()
-    ].tolist()
+    train_images = np.array(coco_dict["images"])[(np.array(train_indices) - 1).tolist()].tolist()
+    val_images = np.array(coco_dict["images"])[(np.array(val_indices) - 1).tolist()].tolist()
     # divide annotations
     train_annotations = list()
     val_annotations = list()
@@ -1932,10 +1838,7 @@ def add_bbox_and_area_to_coco(
         # assign annotation bbox
         if add_bbox:
             coco_polygons = []
-            [
-                coco_polygons.extend(coco_polygon)
-                for coco_polygon in annotation["segmentation"]
-            ]
+            [coco_polygons.extend(coco_polygon) for coco_polygon in annotation["segmentation"]]
             minx, miny, maxx, maxy = list(
                 [
                     min(coco_polygons[0::2]),
@@ -1954,9 +1857,7 @@ def add_bbox_and_area_to_coco(
 
         # assign annotation area
         if add_area:
-            shapely_multipolygon = get_shapely_multipolygon(
-                coco_segmentation=annotation["segmentation"]
-            )
+            shapely_multipolygon = get_shapely_multipolygon(coco_segmentation=annotation["segmentation"])
             annotations[ind]["area"] = shapely_multipolygon.area
 
     coco_dict["annotations"] = annotations
@@ -1999,9 +1900,7 @@ def count_images_with_category(coco_file_path):
     for annotation in coco["annotations"]:
         image_id = annotation["image_id"]
         cid = annotation["category_id"]
-        image_id_2_category_2_count[image_id][cid] = (
-            image_id_2_category_2_count[image_id][cid] + 1
-        )
+        image_id_2_category_2_count[image_id][cid] = image_id_2_category_2_count[image_id][cid] + 1
 
     category_2_count = defaultdict(lambda: 0)
     for image_id, image_category_2_count in image_id_2_category_2_count.items():
@@ -2059,9 +1958,7 @@ class CocoVid:
             category: CocoCategory
         """
 
-        assert (
-            type(category) == CocoCategory
-        ), "category must be a CocoCategory instance"
+        assert type(category) == CocoCategory, "category must be a CocoCategory instance"
 
         self.categories.append(category)
 
